from tcod.context import Context
from tcod.console import Console
from tcod.map import compute_fov

import entity_types
from floor_map import FloorMap #TODO: replace with "DungeonMap"

class Engine:
	def __init__(self, floor_map: FloorMap):
		from event_handler import InGameEventHandler
		self.event_handler = InGameEventHandler(self)
		self.floor_map = floor_map
		self.floor_map.engine = self #references everywhere!

		#grab the player object
		self.player = self.floor_map.player

		#kick off the render
		self.update_fov()

	def run_loop(self, context: Context, console: Console) -> None:
		while True:
			if self.event_handler.handle_events():
				self.handle_entities()

			self.handle_rendering(context, console)

	def handle_entities(self) -> None:
		self.update_fov() #knowing the FOV lets entities mess with it

		#all *actors* in the level
		for actor in set(self.floor_map.actors) - {self.player}:
			if actor.ai:
				actor.ai.apply()

	def handle_rendering(self, context: Context, console: Console) -> None:
		#map and all entities within
		self.floor_map.render(console)

		#UI
		console.print(
			x=1, y=47,
			string=f"HP: {self.player.fighter.current_hp}/{self.player.fighter.current_hp}",
		)

		#send to the screen
		context.present(console)
		console.clear()

	#utils
	def update_fov(self):
		self.floor_map.visible[:] = compute_fov(
			self.floor_map.tiles["transparent"],
			(self.player.x, self.player.y),
			radius = 8,
		)

		#add the visible tiles to the explored list
		self.floor_map.explored |= self.floor_map.visible
