from __future__ import annotations

import copy
from typing import Optional, Tuple, Type

from components.base_ai import BaseAI
from components.fighter import Fighter

class Entity:
	def __init__(
		self,
		x: int = 0,
		y: int = 0,
		char: str = "?",
		color: Tuple[int, int, int] = (255, 255, 255),
		name: str = "<Unnamed>",
		walkable: bool = True,
		floor_map = None,
	):
		self.x = x
		self.y = y
		self.char = char
		self.color = color
		self.name = name
		self.walkable = walkable
		self.floor_map = floor_map

	def spawn(self, x: int, y: int, floor_map):
		clone = copy.deepcopy(self)
		clone.x = x
		clone.y = y
		clone.floor_map = floor_map
		clone.floor_map.entities.add(clone)
		return clone

	def set_pos(self, x: int, y: int) -> None:
		self.x = x
		self.y = y


#actors are entities that can act on their own
class Actor(Entity):
	def __init__( #yep, this is ugly
		self,
		x: int = 0,
		y: int = 0,
		char: str = "?",
		color: Tuple[int, int, int] = (255, 255, 255),
		name: str = "<Unnamed>",
		walkable: bool = True,
		floor_map = None,

		#actor-specific stuff
		ai_class: Type[BaseAI] = None,
		fighter: Fighter = None,
	):
		super().__init__(
			x = x,
			y = y,
			char = char,
			color = color,
			name = name,
			walkable = walkable,
			floor_map = floor_map,
		)

		self.ai: Optional[BaseAI] = ai_class(self)
		self.fighter = fighter
		self.fighter.entity = self

	def is_alive(self) -> bool:
		return bool(self.ai)
